-- RiksTradeStacker
if (RTS == nil) then RTS = {} end;
if (RTS.vars == nil) then RTS.varsDefault(); end;
local log = RTS.log

RTS.addonName = "RiksTradeStacker"

-- ************************************************************************
local function AttemptStacking(where)
  assert(where == BAG_BACKPACK or where == BAG_BANK, 
         "Invalid target, should be either bag or bank");
         
  if (where == BAG_BACKPACK and RTS.vars.stackInBackpack == false) then return end;
  if (where == BAG_BANK and RTS.vars.stackInBank == false) then return end;
         
  local target = "backpack";
  if (where == BAG_BANK) then
    target = "bank";
  end
			
	-- -- Get total number of slots in the backpack
  local bagSlots = GetBagSize(where)
	local stackableItems = {}
	
	for currSlot = 0, bagSlots - 1 do
		-- Check current slot
		local stack, maxStack = GetSlotStackSize(where, currSlot)
		
		-- Do we have an item, and the stack is not full (items that do not stack
    -- have stack == maxStack == 1)
		if (stack > 0 and stack < maxStack) then
		
			local id = GetItemInstanceId(where, currSlot)
		
			-- Check whether we have already information about
			-- a stack of this item 
			if (stackableItems[id] == nil) then
				-- Nope, no information as of now, create our information
				-- 
				stackableItems[id] = {}
				stackableItems[id].slot = currSlot
				stackableItems[id].free = maxStack - stack
			else
				-- We have this item already in the inventory				
        local ok
				if (CallSecureProtected("PickupInventoryItem", where, currSlot, stack)) then
          ok = CallSecureProtected("PlaceInInventory", where, stackableItems[id].slot)
          
          local itemLink = GetItemLink(where, currSlot, LINK_STYLE_BRACKETS)
          if (ok) then          
            if (stack == stackableItems[id].free) then
              -- The number of items we moved equals the free spaces
              stackableItems[id] = nil
              log("Stacked " .. stack .. " " .. itemLink .. " in " .. target .. ", stack is now full")
              
            elseif (stack < stackableItems[id].free) then
              -- We moved less items than there was space available.
              stackableItems[id].free = stackableItems[id].free - stack
              log("Stacked " .. stack .. " " .. itemLink .. " in " .. target .. ", stack has space for " .. stackableItems[id].free .. " more")
              
            else                     
              -- We tried to move more items than there was space available.
              -- The remaining items form the new slot for this item
              local moved = stackableItems[id].free
              local remaining = stack - moved
              local free = maxStack - stack + stackableItems[id].free
              
              stackableItems[id].slot = currSlot
              stackableItems[id].free = free
              
              log("Fill existing stack in " .. target .. " with " .. moved .. " " .. itemLink .. ", remaining stack has space for " .. free .. " more")
            end
            
          else
            log("Failed to move " .. stack .. " " .. itemLink .. " in " .. target)
          end            
        end  
			end
      
      ClearCursor()
		end
	end
  ClearCursor()
end

-- ************************************************************************
function RTS.AttemptStackingBag()
  AttemptStacking(BAG_BACKPACK);
end

-- ************************************************************************
function RTS.AttemptStackingBank()
  zo_callLater(function() AttemptStacking(BAG_BANK) end, 250);
end

-- ************************************************************************
function RTS.OnLoad(eventCode, addOnName)
	if addOnName ~= RTS.addonName then return end
  
  local varsVersion = 2
  RTS.vars = ZO_SavedVars:NewAccountWide("Riks_TS_Vars", varsVersion, nil, RTS.vars)
	
	-- Add slash commands
  RTS.CreateSlashCommands()
	
	-- Event registration	
  EVENT_MANAGER:RegisterForEvent(RTS.addonName, EVENT_TRADE_SUCCEEDED, RTS.AttemptStackingBag)
	EVENT_MANAGER:RegisterForEvent(RTS.addonName, EVENT_MAIL_TAKE_ATTACHED_ITEM_SUCCESS, RTS.AttemptStackingBag)
  EVENT_MANAGER:RegisterForEvent(RTS.addonName, EVENT_OPEN_BANK, RTS.AttemptStackingBank);  
  
  -- Create the menu
  RTS.CreateMenu()
	
	EVENT_MANAGER:UnregisterForEvent(RTS.addonName, EVENT_ADD_ON_LOADED)
end

-- ************************************************************************
EVENT_MANAGER:RegisterForEvent(RTS.addonName, EVENT_ADD_ON_LOADED, RTS.OnLoad)